#ifndef TMOUSE_H
#define TMOUSE_H

#include  <dos.h>

#define   MOUSECALL int86(0x33, &r, &r)
#define   RESET_MOUSE() INITIALIZE_MOUSE()
union REGS r;

int       MOUSE = 0, MOUSE_MOVED = 0;
int       MOUSE_X, MOUSE_Y;
int       MOUSE_LEFT_LIM, MOUSE_RIGHT_LIM;
int       MOUSE_TOP_LIM, MOUSE_BOTTOM_LIM;
int       SHIFT_BITS;
int       MOUSE_MAX_COLUMNS;
int       MOUSE_MAX_ROWS = 24;

void      INITIALIZE_MOUSE(void) {
          int version;

          r.x.ax = 0x0f00;
          int86(0x10, &r, &r);
          MOUSE_MAX_COLUMNS = r.h.ah - 1;
          MOUSE_BOTTOM_LIM = 24;
          MOUSE_LEFT_LIM = 0;
          MOUSE_TOP_LIM = 0;
          MOUSE_RIGHT_LIM = MOUSE_MAX_COLUMNS;
          SHIFT_BITS = (MOUSE_RIGHT_LIM == 39) ? 4 : 3;

          r.x.ax = 0x3001;
          int86(0x21, &r, &r);
          version = r.h.al << 8 | r.h.ah;
          if (version < 0x0314) {  /* VERSION LESS THAN 3.20 */
               MOUSE = 0;
               return;
          }
          r.x.ax = 0x0000;
          MOUSECALL;
          MOUSE = r.x.ax;
}

void      DEFINE_MOUSE(char character, char color) {
          if (MOUSE) {
               if (character == '\0') {
                    COLOR_MOUSE(color);
               } else {
                    r.x.ax = 0x000a;
                    r.x.bx = 0x0000;
                    r.x.cx = 0x0000;
                    r.h.dh = color;
                    r.h.dl = character;
                    MOUSECALL;
               }
          }
}

void      COLOR_MOUSE(char color) {
          if (MOUSE) {
               r.x.ax = 0x000a;
               r.x.bx = 0x0000;
               r.x.cx = 0x00ff;
               r.h.dl = 0x00;
               r.h.dh = color;
               MOUSECALL;
          }
}

void      LIMIT_MOUSE(int top, int left, int length, int width) {
          if (! MOUSE) return;

          MOUSE_LEFT_LIM = left;
          MOUSE_RIGHT_LIM = left + width;
          MOUSE_TOP_LIM = top;
          MOUSE_BOTTOM_LIM = top + length;

          if (MOUSE_RIGHT_LIM > MOUSE_MAX_COLUMNS)
               MOUSE_RIGHT_LIM = MOUSE_MAX_COLUMNS;
          if (MOUSE_BOTTOM_LIM > MOUSE_MAX_ROWS)
               MOUSE_BOTTOM_LIM = MOUSE_MAX_ROWS;

          r.x.ax = 0x0007;
          r.x.cx = left << SHIFT_BITS;
          r.x.dx = MOUSE_RIGHT_LIM << SHIFT_BITS;
          MOUSECALL;
          r.x.ax = 0x0008;
          r.x.cx = top << 3;
          r.x.dx = MOUSE_BOTTOM_LIM << 3;
          MOUSECALL;
}

void      MOUSE_ON(void) {
          int i;

          if (MOUSE)
               for (i=0; i<20; i++) {
                    r.x.ax = 0x0001;
                    MOUSECALL;
               }
}

void      MOUSE_OFF(void) {
          if (MOUSE) {
               r.x.ax = 0x0002;
               MOUSECALL;
          }
}

int       CLICK_AT(void) {
          int s;

          if (! MOUSE) return;
          MOUSE_MOVED = 0;
          r.x.ax = 0x0003;
          MOUSECALL;
          if (r.x.bx == 0) return 0;
          r.x.ax = 0x0006;
          r.x.bx = 0x0000;
          MOUSECALL;
          while (r.x.ax) {
               r.x.ax = 0x0006;
               r.x.bx = 0x0000;
               MOUSECALL;
          }
          MOUSE_MOVED = 1;
          MOUSE_X = (r.x.cx >> SHIFT_BITS) - MOUSE_LEFT_LIM;
          MOUSE_Y = (r.x.dx >> 3) - MOUSE_TOP_LIM;

          return r.x.bx;
}

#endif
